import jakarta.mail.Message
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import tangram.lib.data.DataBuffer
import tangram.lib.db.Cmd
import tangram.lib.files.CSVReader
import tangram.lib.mail.MailReader

import java.nio.file.Files
import java.nio.file.Path
import java.nio.file.StandardCopyOption
import java.util.zip.ZipEntry
import java.util.zip.ZipFile

// start skriptu
new KoposStav().run()

/*
Import stavu zasob z centralneho skladu Kopos Kolin
Stav sa pravidelne exportuje a zasiela mailom, v prilohe mailu je subor "kolinstav.zip"
Vnutri zip suboru je "kolinstav.csv" stlpcami:
- Item-id = interny kod polozky v Koline (nepouzivame)
- Suma = disponibilny stav zasob (Suma = FyzZas-Objednavky)
- FyzZas = uctovny stav zasob (nepouzivame)
- Objednavky = rozpracovane objednavky (nepouzivame)
- BarCode = ciarovy kod, podla ktore parujeme s nasou databazou
Po nacitani suboru sa mail vymaze
Ak sa najde viac emailov s prilohou "kolinstav.zip", pouzije sa len najnovsi a vsetky sa vymazu
*/
class KoposStav
{
    private final Logger log = LoggerFactory.getLogger(this.class.getName());

    // konfiguracia POP servera
    private final MailReader mail = new MailReader()
            .setHost("mail.mojhosting.sk")
            .setPort(110)
            .setTLS(false)
            .setLogin("stavd365@kopos.sk")
            .setPassword("KheLcEsQ7qV26");
    // pomocny adresar, do ktoreho sa ukladaju subory z prilohy mailu
    private final tempdir = "temp"
    // nazov prilohy
    private final String zipfilename = "kolinstav.zip"
    // nazov CSV suboru vnutri zipu
    private final String csvfilename = "kolinstav.csv"
    // nazov pomocneho suboru po nacitani z emailu
    private final String tempfilename = Path.of(tempdir, zipfilename).toString()
    // skod skladu, do ktoreho sa uklada dispozicny stav
    private final String sklad = "XXX"

    // Navratova hodnota sa prenesie ako vysledok skriptu
    public String run ()
    {
        log.info("KOPOS - import stavu zasob z centraly Kolin")

        // nacitanie e-mailov s prilohou "kolinstav.zip"
        if (!readMail())
        {
            log.info("Ziaden email s prilohou nebol najdeny")
            return ("NO EMAIL FOUND")
        }

        // otvorenie zip suboru, import do databazy
        importZip()

        log.info("KOPOS - import dokonceny")
        return "IMPORT OK"
    }

    /**
     * Import ZIP suboru
     * @return pocet importovanych zaznamov
     */
    private void importZip()
    {
        // Otvorenie ZIP suboru, vyhladanie CSV vnutri zipu
        log.debug("Otvorenie ZIP suboru " + tempfilename)
        try (ZipFile zip = new ZipFile(tempfilename))
        {
            // Vyhladame CSV suboru vnutri ZIPu
            ZipEntry entry = zip.getEntry(csvfilename)
            if (entry==null)
                throw new IOException("Subor " + csvfilename + " nebol najdeny vnutri ZIP suboru")
            // Otvorenie streamu z CSV suboru
            try (InputStream stream = zip.getInputStream(entry))
            {
                importCSV (stream)
            }
        }
    }

    /**
     * IMPORT CSV suboru
     * @param stream otvoreny CSV subor (v tomto pripade Stream zo Zipu)
     * @return pocet importovanych riadkov
     */
    private void importCSV (InputStream stream)
    {
        // Vymazanie doterajsieho stavu z databazy
        log.debug("Pripojenie k databaze a vymazanie predchadzajuceho stavu")
        int deleted = Cmd.execute("DELETE FROM SKDISP WHERE sklad='$sklad'")
        log.debug("Vymazany predchadzajuci stav, pocet zaznamov $deleted")

        // Import CSV po riadkoch
        log.debug("Import CSV suboru")
        int countIns = 0
        int countskip = 0
        try (CSVReader reader = new CSVReader(stream))
        {
            reader.setDecimalPoint('.' as char);
            reader.setItemDelimiter(';' as char);
            reader.skipHeader();
            while (reader.readln())
            {
                String itemID = reader.getStr(1)
                BigDecimal stav = reader.getNum(2)
                String barcode = reader.getStr(5)
//                log.debug("ROW " + reader.getRownum()
//                        + " ITEM=" + itemID
//                        + " STAV=" + stav
//                        + " EAN=" + barcode)
                // Hladame ciarovy kod v databaze
                String kod = Cmd.selectString("SELECT max(kod) FROM TOVBARCODE WHERE barcode='$barcode' and hlavny is not null")
                if (kod)
                {
                    DataBuffer data = new DataBuffer()
                    data.put('sklad', sklad)
                    data.put('kod', kod)
                    data.put('stav', stav)
                    data.put('disp', stav)
                    Cmd.insert('SKDISP', data)
                    countIns++
                }
                else
                {
                    log.error("Neznamy ciarovy kod $barcode v riadku ${reader.getRownum()}, item=$itemID")
                    countskip++
                }

            }
        }
        log.debug("Import CSV dokonceny, vlozene $countIns, vynechane $countskip")
    }

    /*
         Nacitanie emailov.
         Ak ma email prilohu "kolinstav.zip", tak sa ulozi ako subor do pod-adresara "temp", sprava sa vymaze
         Ak sa najde viac sprav s prilohou, ulozi sa len posledna verzia suboru, tieto spravy sa vsetky vymazu
         Spravy, ktore nemaju spravnu prilohu, sa nevymazavaju
         Funkcia vrati true=nasiel sa mail alebo false=ziaden vhodny mail
    */
    private boolean readMail ()
    {
        log.info("Connecting to MAIL server " + mail);
        boolean found = false

        try
        {
            // pripojenie na server
            mail.open();
            log.info("Mail server connected");
            // nacitanie vsetkych sprav do pola
            Message[] messages = mail.getMessages();
            for (int i = 0; i < messages.length; i++)
            {
                // Sprava
                Message m = mail.getMessage(i);
                log.info(mail.getMessageInfo(i));
                // Zoznam priloh - nazvy suborov
                var fileNames = mail.getFiles(i);
                log.debug("ATTACHMENTS " + fileNames);
                // Ak priloha obsahuje "kolinstav.zip", tak sa ulozi do TEMP adresara  (nazov je case-sensitive !)
                if (fileNames.contains(zipfilename))
                {
                    log.debug("SAVING ATTACHMENT TO FILE " + tempfilename);
                    try (var input = mail.getStream(i, zipfilename)) {
                        Files.copy(input, Path.of(tempfilename), StandardCopyOption.REPLACE_EXISTING);
                    }
                    found = true
                    // Vymazanie spravy
                    // log.info("DELETING MESSAGE " + m.getMessageNumber());
                    // mail.deleteMessage(i);
                }
            }
        }
        finally
        {
            mail.close();
            log.info("MAIL server disconnected");
        }

        return found;
    }
}