import com.fasterxml.jackson.annotation.JsonIgnoreProperties
import com.fasterxml.jackson.dataformat.xml.annotation.JacksonXmlElementWrapper
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import tangram.lib.data.*
import tangram.lib.db.Cmd
import tangram.lib.files.FileUtils

import java.time.LocalDate

// start skriptu, argument "a1" musi obsahovat adresar v ktorom su subory
// new PpgSapInvoice().run(binding)
PpgSapInvoice.start(binding)

// Import XML suboru "SAP invoice", ktory sa naiportuje ako prijemka do skladu
class PpgSapInvoice
{
    private static final Logger log = LoggerFactory.getLogger(PpgSapInvoice.class);

    // Navratova hodnota sa prenesie ako vysledok skriptu
    public static String start(Binding binding)
    {
        // Zoznam suborov zo zadaneho adresara
        String directory = Convert.toString(binding.getVariable('a1'))
        log.info("PPG SAP import, adresar=${directory}, subory=*.XML")
        File[] files = FileUtils.fileList(directory, "*.XML")
        log.debug("Najdene subory: " + files.length)
        if (files.length == 0) return "NO FILES TO IMPORT"

        // Import suborov po jednom, po uspesnom import sa subor vymaze
        for (File f : files)
        {
            importFile(f)
            f.delete()
        }

        return "IMPORT OK"
    }

    // Import jedneho suboru do databazy, vracia true=naimportovane alebo false=neimportovane
    private static void importFile (File file)
    {
        log.info("PPG - import SAP invoice from " + file.getAbsolutePath())

        // XML parsing, data su nacitane do objektu so strukturou Invoice
        Invoice invoice = XML.readFile(file.getPath(), Invoice.class)

        // LOG - obsah XML suboru
        log.debug(invoice.toString())
        for (def line : invoice.invoice_line)
            log.debug(line.toString())

        // Hlavicka dokladu
        DataBuffer header = createHeader(invoice, file.getName())
        String headerID = header.getString('xx')
        Cmd.insert('EXTDOKH', header)

        // Riadky dokladu
        for (def line : invoice.invoice_line)
            Cmd.insert('EXTDOKR', createLine(line, headerID, invoice.sap_order_id))

        log.info("SAP file imported into ${header.druhdok}/${header.cislo}")
    }

    // Vytvorenie hlavicky Externeho dokladu
    private static DataBuffer createHeader (Invoice invoice, String filename)
    {
        DataBuffer data = new DataBuffer()
        data.xx = Cmd.createXX()
        data.pda = 'SAP'
        data.stav = '1'     // ukonceny doklad
        data.datum = LocalDate.now()
        data.typ = 'A'  // Prijem
        data.druhdok = 'PRSAP'
        data.cislo = invoice.invoice_id;   // Cmd.genIdMaxPlus("EXTDOKH", "cislo", 10);
        data.poznamka = invoice.sap_order_id + " " + filename;
        return data
    }

    // Vytvorenie riadka Externeho dokladu
    private static DataBuffer createLine (InvoiceLine line, String headerID, String orderID)
    {
        DataBuffer data = new DataBuffer()
        data.xx = Cmd.createXX()
        data.xh = headerID;
        data.xn = Convert.toInt(line.sap_line_id)
        data.kod = Str.padLeft(line.product_id, 8, '0' as char)
        data.mn = line.delivered_qty
        data.cenabd = Num.div(line.price_list, line.delivered_qty, 5)
        data.poznamka = 'SUMA=' + line.price_list + " DPH=" + line.tax
        data.objcis = orderID
        return data
    }

    // DTO - hlavicka faktury
    @JsonIgnoreProperties(ignoreUnknown = true)
    static class Invoice
    {
        public String sap_order_id;
        public String invoice_id;
        // nasleduje "unwrapped" pole, t.j. nie je vlozene do samostatneho tagu
        @JacksonXmlElementWrapper(useWrapping = false)
        public ArrayList<InvoiceLine> invoice_line = new ArrayList<>();

        @Override
        public String toString() {
            return "Invoice" +
                    " sap_order_id=" + sap_order_id +
                    " invoice_id=" + invoice_id +
                    " invoice_line=" + invoice_line.size() ;
        }
    }

    // DTO - riadok faktury
    @JsonIgnoreProperties(ignoreUnknown = true)
    static class InvoiceLine
    {
        public String sap_line_id;
        public String product_id;
        public BigDecimal delivered_qty;
        public BigDecimal price_list;
        public BigDecimal tax;

        @Override
        public String toString() {
            return "> sap_line_id=" + sap_line_id +
                    " product_id=" + product_id +
                    " delivered_qty=" + delivered_qty +
                    " price_list=" + price_list +
                    " tax=" + tax;
        }
    }
}